@@ -81,11 +81,14 @@ BATCH_PREDICTION_PATH = 'batchprediction'
 CLUSTER_PATH = 'cluster'
 CENTROID_PATH = 'centroid'
 BATCH_CENTROID_PATH = 'batchcentroid'
+ANOMALY_PATH = 'anomaly'
+ANOMALY_SCORE_PATH = 'anomalyscore'
+BATCH_ANOMALY_SCORE_PATH = 'batchanomalyscore'
 
 
 # Resource Ids patterns
 ID_PATTERN = '[a-f0-9]{24}'
-SHARED_PATTERN = '[a-zA-Z0-9]{27}'
+SHARED_PATTERN = '[a-zA-Z0-9]{24,30}'
 SOURCE_RE = re.compile(r'^%s/%s$' % (SOURCE_PATH, ID_PATTERN))
 DATASET_RE = re.compile(r'^(public/)?%s/%s$|^shared/%s/%s$' % (
     DATASET_PATH, ID_PATTERN, DATASET_PATH, SHARED_PATTERN))
@@ -101,6 +104,10 @@ CLUSTER_RE = re.compile(r'^(public/)?%s/%s$|^shared/%s/%s$' % (
 CENTROID_RE = re.compile(r'^%s/%s$' % (CENTROID_PATH, ID_PATTERN))
 BATCH_CENTROID_RE = re.compile(r'^%s/%s$' % (BATCH_CENTROID_PATH,
                                              ID_PATTERN))
+ANOMALY_RE = re.compile(r'^%s/%s$' % (ANOMALY_PATH, ID_PATTERN))
+ANOMALY_SCORE_RE = re.compile(r'^%s/%s$' % (ANOMALY_SCORE_PATH, ID_PATTERN))
+BATCH_ANOMALY_SCORE_RE = re.compile(r'^%s/%s$' % (BATCH_ANOMALY_SCORE_PATH,
+                                                  ID_PATTERN))
 
 
 RESOURCE_RE = {
@@ -113,7 +120,13 @@ RESOURCE_RE = {
     'batchprediction': BATCH_PREDICTION_RE,
     'cluster': CLUSTER_RE,
     'centroid': CENTROID_RE,
-    'batchcentroid': BATCH_CENTROID_RE}
+    'batchcentroid': BATCH_CENTROID_RE,
+    'anomaly': ANOMALY_RE,
+    'anomalyscore': ANOMALY_SCORE_RE,
+    'batchanomalyscore': BATCH_ANOMALY_SCORE_RE}
+
+PREDICTIONS_RE = []
+
 DOWNLOAD_DIR = '/download'
 
 # Headers
@@ -268,23 +281,36 @@ def get_batch_centroid_id(batch_centroid):
     return get_resource(BATCH_CENTROID_RE, batch_centroid)
 
 
+def get_anomaly_id(anomaly):
+    """Returns an anomaly/id.
+
+    """
+    return get_resource(ANOMALY_RE, anomaly)
+
+
+def get_anomaly_score_id(anomaly_score):
+    """Returns an anomalyscore/id.
+
+    """
+    return get_resource(ANOMALY_SCORE_RE, anomaly_score)
+
+
+def get_batch_anomaly_score_id(batch_anomaly_score):
+    """Returns a batchanomalyscore/id.
+
+    """
+    return get_resource(BATCH_ANOMALY_SCORE_RE, batch_anomaly_score)
+
+
 def get_resource_id(resource):
     """Returns the resource id if it falls in one of the registered types
 
     """
     if isinstance(resource, dict) and 'resource' in resource:
         return resource['resource']
-    elif isinstance(resource, basestring) and (
-            SOURCE_RE.match(resource)
-            or DATASET_RE.match(resource)
-            or MODEL_RE.match(resource)
-            or PREDICTION_RE.match(resource)
-            or EVALUATION_RE.match(resource)
-            or ENSEMBLE_RE.match(resource)
-            or BATCH_PREDICTION_RE.match(resource)
-            or CLUSTER_RE.match(resource)
-            or CENTROID_RE.match(resource)
-            or BATCH_CENTROID_RE.match(resource)):
+    elif isinstance(resource, basestring) and any(
+            resource_re.match(resource) for _, resource_re
+            in RESOURCE_RE.items()):
         return resource
     else:
         return
@@ -339,11 +365,11 @@ def check_resource(resource, get_method, query_string='', wait_time=1,
 
     """
     def get_kwargs(resource_id):
-        if not (EVALUATION_RE.match(resource_id) or
-                PREDICTION_RE.match(resource_id) or
-                BATCH_PREDICTION_RE.match(resource_id) or
-                CENTROID_RE.match(resource_id) or
-                BATCH_CENTROID_RE.match(resource_id)):
+        no_qs = [EVALUATION_RE, PREDICTION_RE, BATCH_PREDICTION_RE,
+                 CENTROID_RE, BATCH_CENTROID_RE, ANOMALY_SCORE_RE,
+                 BATCH_ANOMALY_SCORE_RE]
+        if not (any(resource_re.match(resource_id) for
+                    resource_re in no_qs)):
             return {'query_string': query_string}
         return {}
 
@@ -536,7 +562,10 @@ class BigML(object):
             'batchprediction': self.get_batch_prediction,
             'cluster': self.get_cluster,
             'centroid': self.get_centroid,
-            'batchcentroid': self.get_batch_centroid}
+            'batchcentroid': self.get_batch_centroid,
+            'anomaly': self.get_anomaly,
+            'anomalyscore': self.get_anomaly_score,
+            'batchanomalyscore': self.get_batch_anomaly_score}
 
     def _set_api_urls(self, dev_mode=False, domain=None):
         """Sets the urls that point to the REST api methods for each resource
@@ -574,6 +603,9 @@ class BigML(object):
         self.cluster_url = self.url + CLUSTER_PATH
         self.centroid_url = self.url + CENTROID_PATH
         self.batch_centroid_url = self.url + BATCH_CENTROID_PATH
+        self.anomaly_url = self.url + ANOMALY_PATH
+        self.anomaly_score_url = self.url + ANOMALY_SCORE_PATH
+        self.batch_anomaly_score_url = self.url + BATCH_ANOMALY_SCORE_PATH
 
 
     def _create(self, url, body, verify=None):
@@ -1105,7 +1137,8 @@ class BigML(object):
                     or MODEL_RE.match(resource_id)
                     or EVALUATION_RE.match(resource_id)
                     or ENSEMBLE_RE.match(resource_id)
-                    or CLUSTER_RE.match(resource_id)):
+                    or CLUSTER_RE.match(resource_id)
+                    or ANOMALY_RE.match(resource_id)):
                 out.write("%s (%s bytes)\n" % (resource['object']['name'],
                                                resource['object']['size']))
             elif PREDICTION_RE.match(resource['resource']):
@@ -2239,9 +2272,8 @@ class BigML(object):
         if args is not None:
             create_args.update(args)
 
-        model_types = [CLUSTER_PATH]
         origin_resources_checked = self.check_origins(
-            dataset, cluster, create_args, model_types=model_types,
+            dataset, cluster, create_args, model_types=[CLUSTER_PATH],
             wait_time=wait_time, retries=retries)
 
         if origin_resources_checked:
@@ -2304,3 +2336,241 @@ class BigML(object):
         batch_centroid_id = get_batch_centroid_id(batch_centroid)
         if batch_centroid_id:
             return self._delete("%s%s" % (self.url, batch_centroid_id))
+
+    ##########################################################################
+    #
+    # Anomaly detector
+    # https://bigml.com/developers/anomalies
+    #
+    ##########################################################################
+    def create_anomaly(self, datasets, args=None, wait_time=3, retries=10):
+        """Creates an anomaly detector from a `dataset` or a list o `datasets`.
+
+        """
+        create_args = self._set_create_from_datasets_args(
+            datasets, args=args, wait_time=wait_time, retries=retries)
+
+        body = json.dumps(create_args)
+        return self._create(self.anomaly_url, body)
+
+    def get_anomaly(self, anomaly, query_string='',
+                    shared_username=None, shared_api_key=None):
+        """Retrieves an anomaly detector.
+
+           The anomaly parameter should be a string containing the
+           anomaly id or the dict returned by create_anomaly.
+           As the anomaly detector is an evolving object that is processed
+           until it reaches the FINISHED or FAULTY state, the function will
+           return a dict that encloses the model values and state info
+           available at the time it is called.
+
+           If this is a shared anomaly detector, the username and sharing api
+           key must also be provided.
+        """
+        check_resource_type(anomaly, ANOMALY_PATH,
+                            message="A anomaly id is needed.")
+        anomaly_id = get_anomaly_id(anomaly)
+        if anomaly_id:
+            return self._get("%s%s" % (self.url, anomaly_id),
+                             query_string=query_string,
+                             shared_username=shared_username,
+                             shared_api_key=shared_api_key)
+
+    def anomaly_is_ready(self, anomaly, **kwargs):
+        """Checks whether an anomaly detector's status is FINISHED.
+
+        """
+        check_resource_type(anomaly, ANOMALY_PATH,
+                            message="An anomaly id is needed.")
+        resource = self.get_anomaly(anomaly, **kwargs)
+        return resource_is_ready(resource)
+
+    def list_anomalies(self, query_string=''):
+        """Lists all your anomaly detectors.
+
+        """
+        return self._list(self.anomaly_url, query_string)
+
+    def update_anomaly(self, anomaly, changes):
+        """Updates an anomaly detector.
+
+        """
+        check_resource_type(anomaly, ANOMALY_PATH,
+                            message="An anomaly detector id is needed.")
+        anomaly_id = get_anomaly_id(anomaly)
+        if anomaly_id:
+            body = json.dumps(changes)
+            return self._update("%s%s" % (self.url, anomaly_id), body)
+
+    def delete_anomaly(self, anomaly):
+        """Deletes an anomaly detector.
+
+        """
+        check_resource_type(anomaly, ANOMALY_PATH,
+                            message="An anomaly detector id is needed.")
+        anomaly_id = get_anomaly_id(anomaly)
+        if anomaly_id:
+            return self._delete("%s%s" % (self.url, anomaly_id))
+
+
+    ##########################################################################
+    #
+    # Anomaly scores
+    # https://bigml.com/developers/anomalyscores
+    #
+    ##########################################################################
+    def create_anomaly_score(self, anomaly, input_data=None,
+                            args=None, wait_time=3, retries=10):
+        """Creates a new anomaly score.
+
+        """
+        anomaly_score_id = None
+        resource_type = get_resource_type(anomaly)
+        if resource_type == ANOMALY_PATH:
+            anomaly_id = get_anomaly_id(anomaly)
+            check_resource(anomaly_id, self.get_anomaly,
+                           query_string=TINY_RESOURCE,
+                           wait_time=wait_time, retries=retries,
+                           raise_on_error=True)
+        else:
+            raise Exception("An anomaly detector id is needed to create an"
+                            " anomaly score. %s found." % resource_type)
+
+        if input_data is None:
+            input_data = {}
+        create_args = {}
+        if args is not None:
+            create_args.update(args)
+        create_args.update({
+            "input_data": input_data})
+        create_args.update({
+            "anomaly": anomaly_id})
+
+        body = json.dumps(create_args)
+        return self._create(self.anomaly_score_url, body,
+                            verify=self.verify)
+
+    def get_anomaly_score(self, anomaly_score):
+        """Retrieves an anomaly score.
+
+        """
+        check_resource_type(anomaly_score, ANOMALY_SCORE_PATH,
+                            message="An anomaly score id is needed.")
+        anomaly_score_id = get_anomaly_score_id(anomaly_score)
+        if anomaly_score_id:
+            return self._get("%s%s" % (self.url, anomaly_score_id))
+
+    def list_anomaly_scores(self, query_string=''):
+        """Lists all your anomaly_scores.
+
+        """
+        return self._list(self.anomaly_score_url, query_string)
+
+    def update_anomaly_score(self, anomaly_score, changes):
+        """Updates an anomaly_score.
+
+        """
+        check_resource_type(anomaly_score, ANOMALY_SCORE_PATH,
+                            message="An anomaly_score id is needed.")
+        anomaly_score_id = get_anomaly_score_id(anomaly_score)
+        if anomaly_score_id:
+            body = json.dumps(changes)
+            return self._update("%s%s" % (self.url, anomaly_score_id), body)
+
+    def delete_anomaly_score(self, anomaly_score):
+        """Deletes an anomaly_score.
+
+        """
+        check_resource_type(anomaly_score, ANOMALY_SCORE_PATH,
+                            message="An anomaly_score id is needed.")
+        anomaly_score_id = get_anomaly_score_id(anomaly_score)
+        if anomaly_score_id:
+            return self._delete("%s%s" % (self.url, anomaly_score_id))
+
+
+    ##########################################################################
+    #
+    # Batch Anomaly Scores
+    # https://bigml.com/developers/batch_anomalyscores
+    #
+    ##########################################################################
+    def create_batch_anomaly_score(self, anomaly, dataset,
+                                   args=None, wait_time=3, retries=10):
+        """Creates a new batch anomaly score.
+
+
+        """
+        create_args = {}
+        if args is not None:
+            create_args.update(args)
+
+        origin_resources_checked = self.check_origins(
+            dataset, anomaly, create_args, model_types=[ANOMALY_PATH],
+            wait_time=wait_time, retries=retries)
+
+        if origin_resources_checked:
+            body = json.dumps(create_args)
+            return self._create(self.batch_anomaly_score_url, body)
+
+    def get_batch_anomaly_score(self, batch_anomaly_score):
+        """Retrieves a batch anomaly score.
+
+           The batch_anomaly_score parameter should be a string containing the
+           batch_anomaly_score id or the dict returned by
+           create_batch_anomaly_score.
+           As batch_anomaly_score is an evolving object that is processed
+           until it reaches the FINISHED or FAULTY state, the function will
+           return a dict that encloses the batch_anomaly_score values and state
+           info available at the time it is called.
+        """
+        check_resource_type(batch_anomaly_score, BATCH_ANOMALY_SCORE_PATH,
+                            message="A batch anomaly score id is needed.")
+        batch_anomaly_score_id = get_batch_anomaly_score_id(
+            batch_anomaly_score)
+        if batch_anomaly_score_id:
+            return self._get("%s%s" % (self.url, batch_anomaly_score_id))
+
+    def download_batch_anomaly_score(self, batch_anomaly_score, filename=None):
+        """Retrieves the batch anomaly score file.
+
+           Downloads anomaly scores, that are stored in a remote CSV file. If
+           a path is given in filename, the contents of the file are downloaded
+           and saved locally. A file-like object is returned otherwise.
+        """
+        check_resource_type(batch_anomaly_score, BATCH_ANOMALY_SCORE_PATH,
+                            message="A batch anomaly score id is needed.")
+        batch_anomaly_score_id = get_batch_anomaly_score_id(
+            batch_anomaly_score)
+        if batch_anomaly_score_id:
+            return self._download("%s%s%s" % (self.url, batch_anomaly_score_id,
+                                              DOWNLOAD_DIR), filename=filename)
+
+    def list_batch_anomaly_scores(self, query_string=''):
+        """Lists all your batch anomaly scores.
+
+        """
+        return self._list(self.batch_anomaly_score_url, query_string)
+
+    def update_batch_anomaly_score(self, batch_anomaly_score, changes):
+        """Updates a batch anomaly scores.
+
+        """
+        check_resource_type(batch_anomaly_score, BATCH_ANOMALY_SCORE_PATH,
+                            message="A batch anomaly score id is needed.")
+        batch_anomaly_score_id = get_batch_anomaly_score_id(
+            batch_anomaly_score)
+        if batch_anomaly_score_id:
+            body = json.dumps(changes)
+            return self._update("%s%s" % (self.url,
+                                          batch_anomaly_score_id), body)
+
+    def delete_batch_anomaly_score(self, batch_anomaly_score):
+        """Deletes a batch anomaly score.
+
+        """
+        check_resource_type(batch_anomaly_score, BATCH_ANOMALY_SCORE_PATH,
+                            message="A batch anomaly score id is needed.")
+        batch_anomaly_score_id = get_batch_anomaly_score_id(
+            batch_anomaly_score)
+        if batch_anomaly_score_id:
+            return self._delete("%s%s" % (self.url, batch_anomaly_score_id))
